# coding: utf-8

# flake8: noqa
"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from clients.image_gen.models.engine import Engine
from clients.image_gen.models.http_validation_error import HTTPValidationError
from clients.image_gen.models.image_encoding import ImageEncoding
from clients.image_gen.models.image_generation import ImageGeneration
from clients.image_gen.models.image_generation_request import ImageGenerationRequest
from clients.image_gen.models.image_generation_response import ImageGenerationResponse
from clients.image_gen.models.legacy_image_generation_request import (
    LegacyImageGenerationRequest,
)
from clients.image_gen.models.legacy_image_generation_response import (
    LegacyImageGenerationResponse,
)
from clients.image_gen.models.location_inner import LocationInner
from clients.image_gen.models.scheduler import Scheduler
from clients.image_gen.models.sdxl_styles import SDXLStyles
from clients.image_gen.models.validation_error import ValidationError



"""Various utilities."""

import random
import time
from typing import Callable

import httpx


def retry(
    fn: Callable[[], httpx.Response],
    retry_count: int = 5,
    interval: float = 1.0,
    exp_backoff: float = 2.0,
    jitter: float = 1.0,
    maximum_backoff: float = 30.0,
) -> httpx.Response:
    """Retry an HTTP request with exponential backoff and jitter.

    :param fn: function to call
    :type fn: Callable[[], httpx.Response]
    :param retry_count: number of times to retry, defaults to 5
    :type retry_count: int, optional
    :param interval: duration to wait before retry, defaults to 1.0
    :type interval: float, optional
    :param exp_backoff: exponent to increase interval each try, defaults to 2.0
    :type exp_backoff: float, optional
    :param jitter: , defaults to 1.0
    :type jitter: float, optional
    :param maximum_backoff: max duration to wait, defaults to 30.0
    :type maximum_backoff: float, optional
    :raises OctoAIClientError: occurs when a client error is thrown.
    :return: response from api server
    :rtype: httpx.Response
    """
    try:
        resp = fn()
        if retry_count - 1 == 0:
            return resp
        # Raise HTTPStatusError for 4xx or 5xx.
        resp.raise_for_status()
    except httpx.HTTPStatusError:
        if resp.status_code >= 500 or resp.status_code == 429:
            time.sleep(interval + random.uniform(0, jitter))
            return retry(
                fn,
                retry_count - 1,
                interval * exp_backoff,
                exp_backoff,
                jitter,
                maximum_backoff,
            )

        # Raise error on all other client errors.
        elif 400 <= resp.status_code < 499 and resp.status_code != 429:
            # Raise error. Do not retry.
            return resp

    # Raise error without retry on all other exceptions.
    return resp



# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Optional

from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr

from clients.asset_lake.models.storage_descriptor import StorageDescriptor


class VolumeData(BaseModel):
    """
    VolumeData
    """

    asset_type: Optional[Any] = None
    delection_cleanup_done: StrictBool = Field(...)
    mark_for_deletion_time: Optional[StrictStr] = Field(...)
    storage: StorageDescriptor = Field(...)
    version: StrictStr = Field(...)
    __properties = [
        "asset_type",
        "delection_cleanup_done",
        "mark_for_deletion_time",
        "storage",
        "version",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> VolumeData:
        """Create an instance of VolumeData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of storage
        if self.storage:
            _dict["storage"] = self.storage.to_dict()
        # set to None if asset_type (nullable) is None
        # and __fields_set__ contains the field
        if self.asset_type is None and "asset_type" in self.__fields_set__:
            _dict["asset_type"] = None

        # set to None if mark_for_deletion_time (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.mark_for_deletion_time is None
            and "mark_for_deletion_time" in self.__fields_set__
        ):
            _dict["mark_for_deletion_time"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> VolumeData:
        """Create an instance of VolumeData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return VolumeData.parse_obj(obj)

        _obj = VolumeData.parse_obj(
            {
                "asset_type": obj.get("asset_type"),
                "delection_cleanup_done": obj.get("delection_cleanup_done"),
                "mark_for_deletion_time": obj.get("mark_for_deletion_time"),
                "storage": StorageDescriptor.from_dict(obj.get("storage"))
                if obj.get("storage") is not None
                else None,
                "version": obj.get("version"),
            }
        )
        return _obj



from __future__ import annotations

# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum


class SDXLStyles(str, Enum):
    """
    The supported SDXL styles.
    """

    """
    allowed enum values
    """
    BASE = "base"
    ENUM_3D_MINUS_MODEL = "3d-model"
    ANALOG_MINUS_FILM = "analog-film"
    ANIME = "anime"
    CINEMATIC = "cinematic"
    COMIC_MINUS_BOOK = "comic-book"
    CRAFT_CLAY = "Craft Clay"
    MODELING_MINUS_COMPOUND = "modeling-compound"
    DIGITAL_MINUS_ART = "digital-art"
    ENHANCE = "enhance"
    FANTASY_MINUS_ART = "fantasy-art"
    ISOMETRIC = "isometric"
    LINE_MINUS_ART = "line-art"
    LOW_MINUS_POLY = "low-poly"
    NEON_MINUS_PUNK = "neon-punk"
    ORIGAMI = "origami"
    PHOTOGRAPHIC = "photographic"
    PIXEL_MINUS_ART = "pixel-art"
    TILE_MINUS_TEXTURE = "tile-texture"
    ADVERTISING = "Advertising"
    FOOD_PHOTOGRAPHY = "Food Photography"
    REAL_ESTATE = "Real Estate"
    ABSTRACT = "Abstract"
    CUBIST = "Cubist"
    GRAFFITI = "Graffiti"
    HYPERREALISM = "Hyperrealism"
    IMPRESSIONIST = "Impressionist"
    POINTILLISM = "Pointillism"
    POP_ART = "Pop Art"
    PSYCHEDELIC = "Psychedelic"
    RENAISSANCE = "Renaissance"
    STEAMPUNK = "Steampunk"
    SURREALIST = "Surrealist"
    TYPOGRAPHY = "Typography"
    WATERCOLOR = "Watercolor"
    FIGHTING_GAME = "Fighting Game"
    GTA = "GTA"
    SUPER_MARIO = "Super Mario"
    MINECRAFT = "Minecraft"
    POKÉMON = "Pokémon"
    RETRO_ARCADE = "Retro Arcade"
    RETRO_GAME = "Retro Game"
    RPG_FANTASY_GAME = "RPG Fantasy Game"
    STRATEGY_GAME = "Strategy Game"
    STREET_FIGHTER = "Street Fighter"
    LEGEND_OF_ZELDA = "Legend of Zelda"
    ARCHITECTURAL = "Architectural"
    DISCO = "Disco"
    DREAMSCAPE = "Dreamscape"
    DYSTOPIAN = "Dystopian"
    FAIRY_TALE = "Fairy Tale"
    GOTHIC = "Gothic"
    GRUNGE = "Grunge"
    HORROR = "Horror"
    MINIMALIST = "Minimalist"
    MONOCHROME = "Monochrome"
    NAUTICAL = "Nautical"
    SPACE = "Space"
    STAINED_GLASS = "Stained Glass"
    TECHWEAR_FASHION = "Techwear Fashion"
    TRIBAL = "Tribal"
    ZENTANGLE = "Zentangle"
    COLLAGE = "Collage"
    FLAT_PAPERCUT = "Flat Papercut"
    KIRIGAMI = "Kirigami"
    PAPER_MACHE = "Paper Mache"
    PAPER_QUILLING = "Paper Quilling"
    PAPERCUT_COLLAGE = "Papercut Collage"
    PAPERCUT_SHADOW_BOX = "Papercut Shadow Box"
    STACKED_PAPERCUT = "Stacked Papercut"
    THICK_LAYERED_PAPERCUT = "Thick Layered Papercut"
    ALIEN = "Alien"
    FILM_NOIR = "Film Noir"
    HDR = "HDR"
    LONG_EXPOSURE = "Long Exposure"
    NEON_NOIR = "Neon Noir"
    SILHOUETTE = "Silhouette"
    TILT_MINUS_SHIFT = "Tilt-Shift"

    @classmethod
    def from_json(cls, json_str: str) -> SDXLStyles:
        """Create an instance of SDXLStyles from a JSON string"""
        return SDXLStyles(json.loads(json_str))



"""Example OctoAI service scaffold: Flan-T5-Small."""

from transformers import T5ForConditionalGeneration, T5Tokenizer

from octoai.service import Service

"""
Flan-T5 is an instruction-finetuned version of T5, a text-to-text
transformer language model.
"""

_MODEL_NAME = "google/flan-t5-small"
_MODEL_REVISION = "2d036ee774a9cb8d7e03c9f2e78ae0a16343a9d9"


class T5Service(Service):
    """An OctoAI service extends octoai.service.Service."""

    def store_assets(self) -> None:
        """Download model assets."""
        self.setup()

    def setup(self):
        """Download model weights to disk."""
        self.tokenizer = T5Tokenizer.from_pretrained(
            _MODEL_NAME,
            revision=_MODEL_REVISION,
        )
        self.model = T5ForConditionalGeneration.from_pretrained(
            _MODEL_NAME,
            revision=_MODEL_REVISION,
        )

    def infer(self, prompt: str) -> str:
        """Perform inference with the model."""
        inputs = self.tokenizer(prompt, return_tensors="pt")
        outputs = self.model.generate(**inputs)
        response = self.tokenizer.batch_decode(outputs, skip_special_tokens=True)

        return response[0]



# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, List, Optional

from pydantic.v1 import BaseModel, Field, StrictStr, conlist

from clients.asset_lake.models.base_engine import BaseEngine
from clients.asset_lake.models.data_type import DataType
from clients.asset_lake.models.file_format import FileFormat


class TextualInversionData(BaseModel):
    """
    TextualInversionData
    """

    asset_type: Optional[Any] = None
    data_type: Optional[DataType] = None
    engine: BaseEngine = Field(...)
    file_format: Optional[FileFormat] = None
    trigger_words: Optional[conlist(StrictStr)] = None
    __properties = ["asset_type", "data_type", "engine", "file_format", "trigger_words"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TextualInversionData:
        """Create an instance of TextualInversionData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if asset_type (nullable) is None
        # and __fields_set__ contains the field
        if self.asset_type is None and "asset_type" in self.__fields_set__:
            _dict["asset_type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TextualInversionData:
        """Create an instance of TextualInversionData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TextualInversionData.parse_obj(obj)

        _obj = TextualInversionData.parse_obj(
            {
                "asset_type": obj.get("asset_type"),
                "data_type": obj.get("data_type"),
                "engine": obj.get("engine"),
                "file_format": obj.get("file_format"),
                "trigger_words": obj.get("trigger_words"),
            }
        )
        return _obj



# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Optional

from pydantic.v1 import BaseModel


class NoTransferApi(BaseModel):
    """
    This type exists to avoid Optional[TransferApi] so openapi-generator can work.  In this class, type is not added to TransferApiType because it is not a valid TransferApiType. It exists only to note that no TransferApi is defined.  # noqa: E501
    """

    type: Optional[Any] = None
    __properties = ["type"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NoTransferApi:
        """Create an instance of NoTransferApi from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict["type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NoTransferApi:
        """Create an instance of NoTransferApi from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NoTransferApi.parse_obj(obj)

        _obj = NoTransferApi.parse_obj({"type": obj.get("type")})
        return _obj



# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Dict, List, Union

from pydantic.v1 import (
    BaseModel,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
    conlist,
)


class LegacyImageGenerationResponse(BaseModel):
    """
    Legacy model response.  # noqa: E501
    """

    completion: Dict[str, StrictStr] = Field(...)
    prediction_time_ms: Union[StrictFloat, StrictInt] = Field(...)
    replica: StrictStr = Field(...)
    seed: StrictInt = Field(...)
    removed_for_safety: conlist(StrictBool) = Field(...)
    __properties = [
        "completion",
        "prediction_time_ms",
        "replica",
        "seed",
        "removed_for_safety",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LegacyImageGenerationResponse:
        """Create an instance of LegacyImageGenerationResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LegacyImageGenerationResponse:
        """Create an instance of LegacyImageGenerationResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LegacyImageGenerationResponse.parse_obj(obj)

        _obj = LegacyImageGenerationResponse.parse_obj(
            {
                "completion": obj.get("completion"),
                "prediction_time_ms": obj.get("prediction_time_ms"),
                "replica": obj.get("replica"),
                "seed": obj.get("seed"),
                "removed_for_safety": obj.get("removed_for_safety"),
            }
        )
        return _obj



"""Initializes the octoai module."""
from importlib.metadata import version

from . import clients, errors, types, utils

__version__ = version("octoai-sdk")
__all__ = ["clients", "client", "errors", "server", "service", "types", "utils"]



"""Client used to infer from endpoints."""

import json
import logging
import os
import time
from pathlib import Path
from typing import Any, Callable, Dict, Iterator, Mapping, Optional

import httpx
import yaml
from pydantic import BaseModel

import octoai
from octoai import utils
from octoai.errors import OctoAIClientError, OctoAIServerError

LOG = logging.getLogger(__name__)


class _JSONEncoder(json.JSONEncoder):
    def default(self, o: Any) -> Any:
        if isinstance(o, bytes):
            return o.decode()
        return json.JSONEncoder.default(self, o)


class InferenceFuture(BaseModel):
    """Response class for endpoints that support server side async inferences.

    :param response_id: Unique identifier for inference
    :type response_id: str
    :param poll_url: URL to poll status of inference.
    :type poll_url: str
    """

    response_id: str
    poll_url: str


class Client:
    """A class that allows inferences from existing endpoints.

    :param token: api token, defaults to None
    :type token: str, optional
    :param config_path: path to '/.octoai/config.yaml'.  Installed in ~,
        defaults to None and will check home path
    :type config_path: str, optional

    Sets various headers. Gets auth token from environment if none is provided.
    """

    def __init__(
        self,
        token: Optional[str] = None,
        config_path: Optional[str] = None,
    ) -> None:
        """Initialize the :class: `octoai.Client` with an auth token.

        :raises :class:`OctoAIServerError`: server-side failure (unreachable, etc)
        :raises :class:`OctoAIClientError`: client-side failure (throttled, no token)
        """
        token = token if token else os.environ.get("OCTOAI_TOKEN", None)

        if not token:
            # Default path is ~/.octoai/config.yaml for token, can be overridden
            path = Path(config_path) if config_path else Path.home()
            try:
                with open(
                    (path / Path(".octoai/config.yaml")), encoding="utf-8"
                ) as octoai_config_yaml:
                    config_dict = yaml.safe_load(octoai_config_yaml)
                token = config_dict.get("token")
            except FileNotFoundError:
                token = None

        if not token:
            logging.warning(
                "OCTOAI_TOKEN environment variable is not set. "
                + "You won't be able to reach OctoAI endpoints."
            )

        version = octoai.__version__  # type: ignore
        headers = {
            "Content-Type": "application/json",
            "user-agent": f"octoai-{version}",
        }

        if token:
            headers["Authorization"] = f"Bearer {token}"

        # Set all timeouts to 900 seconds to account for cold starts, latency.
        timeout = httpx.Timeout(timeout=900.0)
        self._httpx_client = httpx.Client(timeout=timeout, headers=headers)

    @staticmethod
    def _error(resp: httpx.Response):
        """Raise error of correct type for status code including message and request_id.

        :param response: httpx response

        :raises OctoAIServerError: server-side failures (unreachable, etc)
        :raises OctoAIClientError: client-side failures (throttling, unset token)
        """
        status_code = resp.status_code
        text = resp.text
        req_id_str = ""
        if resp.headers.get("X-User-Request-ID") is not None:
            req_id_str = f'(request_id: {resp.headers.get("X-User-Request-ID")})'
        text += req_id_str
        if status_code >= 500:
            raise OctoAIServerError(f"Server error: {status_code} {text}")
        elif status_code == 429:
            raise OctoAIClientError(f"Rate limit reached error: {status_code} {text}")
        else:
            raise OctoAIClientError(f"Error: {status_code} {text}")

    def infer(self, endpoint_url: str, inputs: Mapping[str, Any]) -> Mapping[str, Any]:
        """Send a request to the given endpoint URL with inputs as request body.

        :param endpoint_url: target endpoint
        :type endpoint_url: str
        :param inputs: inputs for target endpoint
        :type inputs: Mapping[str, Any]

        :raises OctoAIServerError: server-side failures (unreachable, etc)
        :raises OctoAIClientError: client-side failures (throttling, unset token)

        :return: outputs from endpoint
        :rtype: Mapping[str, Any]
        """
        resp = utils.retry(
            lambda: self._httpx_client.post(
                url=endpoint_url,
                headers={"Content-Type": "application/json"},
                content=json.dumps(inputs, cls=_JSONEncoder),
            )
        )
        if resp.status_code != 200:
            self._error(resp)
        return resp.json()

    def infer_async(
        self, endpoint_url: str, inputs: Mapping[str, Any]
    ) -> InferenceFuture:
        """Execute an inference in the background on the server.

        :class:`InferenceFuture` allows you to query status and get results
        once it's ready.

        :param endpoint_url: url to post inference request
        :type endpoint_url: str
        :param inputs: inputs to send to endpoint
        :type inputs: Mapping[str, Any]
        """
        resp = utils.retry(
            lambda: self._httpx_client.post(
                url=endpoint_url, json=inputs, headers={"X-OctoAI-Async": "1"}
            )
        )
        if resp.status_code >= 400:
            self._error(resp)
        resp_json = resp.json()
        future = InferenceFuture(**resp_json)
        return future

    def infer_stream(
        self, endpoint_url: str, inputs: Mapping[str, Any]
    ) -> Iterator[dict]:
        """Stream text event response body for supporting endpoints.

        This is an alternative to loading all response body into memory at once.

        :param endpoint_url: target endpoint
        :type endpoint_url: str
        :param inputs: inputs for target endpoint such as a prompt and other parameters
        :type inputs: Mapping[str, Any]
        :return: Yields a :class:`dict` that contains the server response data.
        :rtype: Iterator[:class:`dict`]
        """
        with self._httpx_client.stream(
            method="POST",
            url=endpoint_url,
            content=json.dumps(inputs, cls=_JSONEncoder),
            headers={"accept": "text/event-stream"},
        ) as resp:
            if resp.status_code >= 400:
                self._error(resp)

            for payload in resp.iter_lines():
                # Empty lines used to separate payloads.
                if payload == "":
                    continue
                # End of stream (OpenAPI /v1/chat/completions).
                elif payload == "data: [DONE]":
                    break
                # Event data identified with "data:"  JSON inside.
                if payload.startswith("data:"):
                    payload_dict = json.loads(payload.lstrip("data:"))
                    yield payload_dict

    def _poll_future(self, future: InferenceFuture) -> Dict[str, str]:
        """Get from poll_url and return response.

        :param future: Future from :meth:`Client.infer_async`
        :type future: :class:`InferenceFuture`
        :raises: :class:`OctoAIClientError`
        :raises: :class:`OctoAIServerError`
        :returns: Dictionary with response
        :rtype: Dict[str, str]
        """
        response = self._httpx_client.get(url=future.poll_url)
        if response.status_code >= 400:
            self._error(response)
        return response.json()

    def is_future_ready(self, future: InferenceFuture) -> bool:
        """Return whether the future's result has been computed.

        This class will raise any errors if the status code is >= 400.

        :param future: Future from :meth:`Client.infer_async`
        :type future: :class:`InferenceFuture`
        :raises: :class:`OctoAIClientError`
        :raises: :class:`OctoAIServerError`
        :returns: True if able to use :meth:`Client.get_future_result`
        """
        resp_dict = self._poll_future(future)
        return "completed" == resp_dict.get("status")

    def get_future_result(self, future: InferenceFuture) -> Optional[Dict[str, Any]]:
        """Return the result of an inference.

        This class will raise any errors if the status code is >= 400.

        :param future: Future from :meth:`Client.infer_async`
        :type future: :class:`InferenceFuture`
        :raises: :class:`OctoAIClientError`
        :raises: :class:`OctoAIServerError`
        :returns: None if future is not ready, or dict of the response.
        :rtype: Dict[str, Any], optional
        """
        resp_dict = self._poll_future(future)
        if resp_dict.get("status") != "completed":
            return None
        response_url = resp_dict.get("response_url")
        response = self._httpx_client.get(response_url)
        if response.status_code >= 400:
            self._error(response)
        return response.json()

    def health_check(self, endpoint_url: str, timeout: float = 900.0) -> int:
        """Check health of an endpoint using a get request.  Try until timeout.

        :param endpoint_url: URL as a str starting with https permitting get requests.
        :type endpoint_url: str
        :param timeout: Seconds before request times out, defaults to 900.
        :type timeout: float
        :return: status code from get request.  200 means ready.
        :rtype: int
        """
        resp = self._health_check(
            lambda: self._httpx_client.get(url=endpoint_url), timeout=timeout
        )
        if resp.status_code >= 400:
            self._error(resp)
        return resp.status_code

    def _health_check(
        self,
        fn: Callable[[], httpx.Response],
        timeout: float = 900.0,
        interval: float = 1.0,
        iteration_count: int = 0,
    ) -> httpx.Response:
        """Check the health of an endpoint.

        :param fn: Get request for endpoint health check.
        :type fn: Callable[[], httpx.Response]
        :param timeout: seconds before health_check times out, defaults to 900.0
        :type timeout: int, optional
        :param interval: seconds to wait before checking endpoint health again,
            defaults to 1.0
        :type interval: int, optional
        :param iteration_count: count total attempts for cold start warning,
            defaults to 0
        :type iteration_count: int
        :raises OctoAIClientError: Client-side failure such as missing api token
        :return: Response once timeout has passed
        :rtype: httpx.Response
        """
        start = time.time()
        try:
            resp = fn()
            if timeout <= 0:
                return resp
            # Raise HTTPStatusError for 4xx or 5xx.
            resp.raise_for_status()
        except httpx.HTTPStatusError:
            if 400 <= resp.status_code < 500:
                # Raise client errors. Do not retry.
                return resp
            if iteration_count == 0 and self.__class__.__name__ == "Client":
                LOG.warning(
                    "Your endpoint may take several minutes to start and be ready to "
                    "serve inferences. You can increase your endpoint's min replicas "
                    "to mitigate cold starts."
                )
            if resp.status_code >= 500:
                stop = time.time()
                current = stop - start
                time.sleep(interval)
                return self._health_check(
                    fn, timeout - current - interval, interval, iteration_count + 1
                )
        # Raise error without retry on all other exceptions.
        return resp



# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import io
import re  # noqa: F401
import warnings
from typing import Any, Optional

from pydantic.v1 import ValidationError, validate_arguments

from clients.image_gen.api_client import ApiClient
from clients.image_gen.api_response import ApiResponse
from clients.image_gen.exceptions import ApiTypeError, ApiValueError  # noqa: F401
from clients.image_gen.models.engine import Engine
from clients.image_gen.models.image_generation_request import ImageGenerationRequest
from clients.image_gen.models.image_generation_response import ImageGenerationResponse
from clients.image_gen.models.legacy_image_generation_request import (
    LegacyImageGenerationRequest,
)
from clients.image_gen.models.legacy_image_generation_response import (
    LegacyImageGenerationResponse,
)


class DefaultApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def generate_images_generate_sd_post(
        self,
        image_generation_request: ImageGenerationRequest,
        force_engine: Optional[Engine] = None,
        **kwargs
    ) -> ImageGenerationResponse:  # noqa: E501
        """Generate Images  # noqa: E501

        Generate images in response to the given request.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.generate_images_generate_sd_post(image_generation_request, force_engine, async_req=True)
        >>> result = thread.get()

        :param image_generation_request: (required)
        :type image_generation_request: ImageGenerationRequest
        :param force_engine:
        :type force_engine: Engine
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ImageGenerationResponse
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the generate_images_generate_sd_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.generate_images_generate_sd_post_with_http_info(
            image_generation_request, force_engine, **kwargs
        )  # noqa: E501

    @validate_arguments
    def generate_images_generate_sd_post_with_http_info(
        self,
        image_generation_request: ImageGenerationRequest,
        force_engine: Optional[Engine] = None,
        **kwargs
    ) -> ApiResponse:  # noqa: E501
        """Generate Images  # noqa: E501

        Generate images in response to the given request.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.generate_images_generate_sd_post_with_http_info(image_generation_request, force_engine, async_req=True)
        >>> result = thread.get()

        :param image_generation_request: (required)
        :type image_generation_request: ImageGenerationRequest
        :param force_engine:
        :type force_engine: Engine
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ImageGenerationResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["image_generation_request", "force_engine"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method generate_images_generate_sd_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("force_engine") is not None:  # noqa: E501
            _query_params.append(("force_engine", _params["force_engine"].value))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["image_generation_request"] is not None:
            _body_params = _params["image_generation_request"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "ImageGenerationResponse",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/generate/sd",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def generate_images_generate_sdxl_post(
        self,
        image_generation_request: ImageGenerationRequest,
        force_engine: Optional[Engine] = None,
        **kwargs
    ) -> ImageGenerationResponse:  # noqa: E501
        """Generate Images  # noqa: E501

        Generate images in response to the given request.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.generate_images_generate_sdxl_post(image_generation_request, force_engine, async_req=True)
        >>> result = thread.get()

        :param image_generation_request: (required)
        :type image_generation_request: ImageGenerationRequest
        :param force_engine:
        :type force_engine: Engine
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ImageGenerationResponse
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the generate_images_generate_sdxl_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.generate_images_generate_sdxl_post_with_http_info(
            image_generation_request, force_engine, **kwargs
        )  # noqa: E501

    @validate_arguments
    def generate_images_generate_sdxl_post_with_http_info(
        self,
        image_generation_request: ImageGenerationRequest,
        force_engine: Optional[Engine] = None,
        **kwargs
    ) -> ApiResponse:  # noqa: E501
        """Generate Images  # noqa: E501

        Generate images in response to the given request.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.generate_images_generate_sdxl_post_with_http_info(image_generation_request, force_engine, async_req=True)
        >>> result = thread.get()

        :param image_generation_request: (required)
        :type image_generation_request: ImageGenerationRequest
        :param force_engine:
        :type force_engine: Engine
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ImageGenerationResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["image_generation_request", "force_engine"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method generate_images_generate_sdxl_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("force_engine") is not None:  # noqa: E501
            _query_params.append(("force_engine", _params["force_engine"].value))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["image_generation_request"] is not None:
            _body_params = _params["image_generation_request"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "ImageGenerationResponse",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/generate/sdxl",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def healthcheck_healthcheck_get(self, **kwargs) -> object:  # noqa: E501
        """Healthcheck  # noqa: E501

        Respond that the server is healthy.  All our setup is done in the lifespan manager prior to yielding, so by the time FastAPI starts taking any requests at all, we are by construction ready to handle them.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.healthcheck_healthcheck_get(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the healthcheck_healthcheck_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.healthcheck_healthcheck_get_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    def healthcheck_healthcheck_get_with_http_info(
        self, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """Healthcheck  # noqa: E501

        Respond that the server is healthy.  All our setup is done in the lifespan manager prior to yielding, so by the time FastAPI starts taking any requests at all, we are by construction ready to handle them.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.healthcheck_healthcheck_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method healthcheck_healthcheck_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "object",
        }

        return self.api_client.call_api(
            "/healthcheck",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def predict_predict_post(
        self, legacy_image_generation_request: LegacyImageGenerationRequest, **kwargs
    ) -> LegacyImageGenerationResponse:  # noqa: E501
        """Predict  # noqa: E501

        Backwards-compatible API for pre-vertical Stable Diffusion XL images.  Note that this endpoint assumes SDXL - there is no support for SD 1.5.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.predict_predict_post(legacy_image_generation_request, async_req=True)
        >>> result = thread.get()

        :param legacy_image_generation_request: (required)
        :type legacy_image_generation_request: LegacyImageGenerationRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: LegacyImageGenerationResponse
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the predict_predict_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.predict_predict_post_with_http_info(
            legacy_image_generation_request, **kwargs
        )  # noqa: E501

    @validate_arguments
    def predict_predict_post_with_http_info(
        self, legacy_image_generation_request: LegacyImageGenerationRequest, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """Predict  # noqa: E501

        Backwards-compatible API for pre-vertical Stable Diffusion XL images.  Note that this endpoint assumes SDXL - there is no support for SD 1.5.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.predict_predict_post_with_http_info(legacy_image_generation_request, async_req=True)
        >>> result = thread.get()

        :param legacy_image_generation_request: (required)
        :type legacy_image_generation_request: LegacyImageGenerationRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(LegacyImageGenerationResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["legacy_image_generation_request"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method predict_predict_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["legacy_image_generation_request"] is not None:
            _body_params = _params["legacy_image_generation_request"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "LegacyImageGenerationResponse",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/predict",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )



from __future__ import annotations

# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum


class DataType(str, Enum):
    """
    DataType
    """

    """
    allowed enum values
    """
    FP16 = "fp16"
    FP32 = "fp32"
    INT4 = "int4"
    INT8 = "int8"

    @classmethod
    def from_json(cls, json_str: str) -> DataType:
        """Create an instance of DataType from a JSON string"""
        return DataType(json.loads(json_str))



# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Optional

from pydantic.v1 import BaseModel, Field

from clients.asset_lake.models.archiver_volume_storage_ref import (
    ArchiverVolumeStorageRef,
)


class StorageDescriptor(BaseModel):
    """
    StorageDescriptor
    """

    archiver_storage_ref: ArchiverVolumeStorageRef = Field(...)
    storage_format: Optional[Any] = Field(...)
    __properties = ["archiver_storage_ref", "storage_format"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StorageDescriptor:
        """Create an instance of StorageDescriptor from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of archiver_storage_ref
        if self.archiver_storage_ref:
            _dict["archiver_storage_ref"] = self.archiver_storage_ref.to_dict()
        # set to None if storage_format (nullable) is None
        # and __fields_set__ contains the field
        if self.storage_format is None and "storage_format" in self.__fields_set__:
            _dict["storage_format"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> StorageDescriptor:
        """Create an instance of StorageDescriptor from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return StorageDescriptor.parse_obj(obj)

        _obj = StorageDescriptor.parse_obj(
            {
                "archiver_storage_ref": ArchiverVolumeStorageRef.from_dict(
                    obj.get("archiver_storage_ref")
                )
                if obj.get("archiver_storage_ref") is not None
                else None,
                "storage_format": obj.get("storage_format"),
            }
        )
        return _obj



"""OctoAI Text Generation."""
from .base_model import BaseModel


class TextGenerationModel(BaseModel):
    """Text generation model."""

    def __init__(self, *args, **kwargs):
        super.__init__(*args, **kwargs)

    def generate(self):
        """Generate text based on request."""
        pass



# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Dict, Optional, Union

from pydantic.v1 import (
    BaseModel,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
    confloat,
    conint,
    constr,
)

from clients.image_gen.models.image_encoding import ImageEncoding
from clients.image_gen.models.scheduler import Scheduler
from clients.image_gen.models.sdxl_styles import SDXLStyles


class ImageGenerationRequest(BaseModel):
    """
    Generate one or more images based on the given parameters.  # noqa: E501
    """

    prompt: constr(strict=True, max_length=10000) = Field(
        ..., description="Text describing the image content to generate."
    )
    prompt_2: Optional[constr(strict=True, max_length=10000)] = Field(
        None,
        description="Text with a high-level description of the image to generate. Used only by SD XL.",
    )
    negative_prompt: Optional[constr(strict=True, max_length=10000)] = Field(
        None, description="Text describing image traits to avoid during generation."
    )
    negative_prompt_2: Optional[constr(strict=True, max_length=10000)] = Field(
        None,
        description="Text with a high level description of things to avoid during generation. Used only by SD XL.",
    )
    checkpoint: Optional[StrictStr] = Field(
        None, description="Checkpoint to be used during image generation"
    )
    textual_inversions: Optional[Dict[str, StrictStr]] = Field(
        None,
        description="A dictionary of textual inversion to be used during image generation. Textual Inversion as key and trigger words as value.",
    )
    loras: Optional[Dict[str, Union[StrictFloat, StrictInt]]] = Field(
        None,
        description="A dictionary of LoRAs to apply. LoRAs as key and its weight (float) as value.",
    )
    sampler: Optional[Scheduler] = None
    height: Optional[StrictInt] = Field(
        None,
        description="Integer representing the height of image to generate. None will default to 512 for SD 1.5 and 1024 for SD XL. Supported resolutions (w,h): SDXL={(1536, 640), (768, 1344), (832, 1216), (1344, 768), (1152, 896), (640, 1536), (1216, 832), (896, 1152), (1024, 1024)}, SD1.5={(512, 512), (1024, 576), (640, 512), (512, 704), (576, 768), (512, 768), (640, 768), (576, 1024), (768, 512)}",
    )
    width: Optional[StrictInt] = Field(
        None,
        description="Integer representing the width of image to generate. None will default to 512 for SD 1.5 and 1024 for SD XL. Supported resolutions (w,h): SDXL={(1536, 640), (768, 1344), (832, 1216), (1344, 768), (1152, 896), (640, 1536), (1216, 832), (896, 1152), (1024, 1024)}, SD1.5={(512, 512), (1024, 576), (640, 512), (512, 704), (576, 768), (512, 768), (640, 768), (576, 1024), (768, 512)}",
    )
    cfg_scale: Optional[
        Union[confloat(le=20.0, strict=True), conint(le=20, strict=True)]
    ] = Field(
        12.0,
        description="Floating-point number represeting how closely to adhere to prompt description. Must be a positive number and less than or equal to 20.0.",
    )
    steps: Optional[conint(strict=True, le=200)] = Field(
        30,
        description="Integer repreenting how many steps of diffusion to run. Must be greater than 0 and less than or equal to 200.",
    )
    num_images: Optional[conint(strict=True, le=10)] = Field(
        1,
        description="Integer representing how many output images to generate with a single prompt/configuration.",
    )
    seed: Optional[StrictInt] = Field(
        None,
        description="Integer number representing the seed of random generators. Fixing random seed is useful when attempting to generate a specific image. Must be greater than 0 and less than 2^32.",
    )
    init_image: Optional[StrictStr] = Field(
        None,
        description="Starting point image encoded in b64 string for Image to Image generation mode.",
    )
    mask_image: Optional[StrictStr] = Field(
        None,
        description="b64 encoded mask image for inpainting. White area should indicate where to paint.",
    )
    strength: Optional[
        Union[confloat(le=1.0, ge=0.0, strict=True), conint(le=1, ge=0, strict=True)]
    ] = Field(
        0.8,
        description="Floating-point number indicating how much creative the Image to Image generation mode should be. Must be greater than 0 and less than or equal to 1.0.",
    )
    style_preset: Optional[SDXLStyles] = None
    use_refiner: Optional[StrictBool] = Field(
        True,
        description="Whether to enable and apply the SDXL refiner model to the image generation.",
    )
    high_noise_frac: Optional[
        Union[confloat(le=1.0, ge=0.0, strict=True), conint(le=1, ge=0, strict=True)]
    ] = Field(
        0.8,
        description="Floating-point number that defines the fraction of steps to perform with the base model. Used only by SD XL. Must be greater than or equal to 0.0 and less than or equal to 1.0.",
    )
    enable_safety: Optional[StrictBool] = Field(
        False,
        description="Boolean defining whether to use safety checker system on generated outputs or not.",
    )
    image_encoding: Optional[ImageEncoding] = None
    force_asset_download: Optional[StrictBool] = Field(
        False,
        description="[Internal] Boolean defining if assets must be re-downloaded into the cache even if present.",
    )
    force_asset_gpu_copy: Optional[StrictBool] = Field(
        False,
        description="[Internal] Boolean defining if assets must to be copied into the GPU even if present.",
    )
    __properties = [
        "prompt",
        "prompt_2",
        "negative_prompt",
        "negative_prompt_2",
        "checkpoint",
        "textual_inversions",
        "loras",
        "sampler",
        "height",
        "width",
        "cfg_scale",
        "steps",
        "num_images",
        "seed",
        "init_image",
        "mask_image",
        "strength",
        "style_preset",
        "use_refiner",
        "high_noise_frac",
        "enable_safety",
        "image_encoding",
        "force_asset_download",
        "force_asset_gpu_copy",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ImageGenerationRequest:
        """Create an instance of ImageGenerationRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ImageGenerationRequest:
        """Create an instance of ImageGenerationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ImageGenerationRequest.parse_obj(obj)

        _obj = ImageGenerationRequest.parse_obj(
            {
                "prompt": obj.get("prompt"),
                "prompt_2": obj.get("prompt_2"),
                "negative_prompt": obj.get("negative_prompt"),
                "negative_prompt_2": obj.get("negative_prompt_2"),
                "checkpoint": obj.get("checkpoint"),
                "textual_inversions": obj.get("textual_inversions"),
                "loras": obj.get("loras"),
                "sampler": obj.get("sampler"),
                "height": obj.get("height"),
                "width": obj.get("width"),
                "cfg_scale": obj.get("cfg_scale")
                if obj.get("cfg_scale") is not None
                else 12.0,
                "steps": obj.get("steps") if obj.get("steps") is not None else 30,
                "num_images": obj.get("num_images")
                if obj.get("num_images") is not None
                else 1,
                "seed": obj.get("seed"),
                "init_image": obj.get("init_image"),
                "mask_image": obj.get("mask_image"),
                "strength": obj.get("strength")
                if obj.get("strength") is not None
                else 0.8,
                "style_preset": obj.get("style_preset"),
                "use_refiner": obj.get("use_refiner")
                if obj.get("use_refiner") is not None
                else True,
                "high_noise_frac": obj.get("high_noise_frac")
                if obj.get("high_noise_frac") is not None
                else 0.8,
                "enable_safety": obj.get("enable_safety")
                if obj.get("enable_safety") is not None
                else False,
                "image_encoding": obj.get("image_encoding"),
                "force_asset_download": obj.get("force_asset_download")
                if obj.get("force_asset_download") is not None
                else False,
                "force_asset_gpu_copy": obj.get("force_asset_gpu_copy")
                if obj.get("force_asset_gpu_copy") is not None
                else False,
            }
        )
        return _obj



"""
test_request.py.

This program makes an example request to your endpoint.
You typically use this program to verify that your service
works correctly:

- After `octoai run` (local development).
  No extra arguments are needed in this case.

  Example:
  `octoai run --command "python test_request.py"`

- After `octoai deploy` (remote deployment).
  Provide your endpoint's host name and port as arguments.

  Example:
  `python test_request.py --endpoint https://myapp123.octoai.cloud`

This program also serves as basic example on how to use the
Python client of the OctoAI SDK. As you develop your service,
you may have to change the inputs in this program to match
what your service expects.
"""
import argparse

from octoai.client import Client

inputs = {"prompt": "Hello world!"}


def main(endpoint):
    """Run inference against the endpoint."""
    # create an OctoAI client
    client = Client()

    # perform inference
    response = client.infer(endpoint_url=f"{endpoint}/infer", inputs=inputs)

    # show the response
    print(response)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--endpoint", type=str, default="http://localhost:8080")

    args = parser.parse_args()
    main(args.endpoint)
